from fedora_messaging import message


class MemberSponsorV1(message.Message):
    """The message sent when a user is added to a group by a sponsor"""

    topic = "fas.group.member.sponsor"
    body_schema = {
        "id": "http://fedoraproject.org/message-schema/noggin",
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "The message sent when a user is added to a group by a sponsor",
        "type": "object",
        "required": ["msg"],
        "properties": {
            "msg": {
                "required": ["agent", "user", "group"],
                "description": "the contents of the event",
                "type": "object",
                "properties": {
                    "agent": {"type": "string"},
                    "user": {"type": "string"},
                    "group": {"type": "string"},
                },
            }
        },
    }


class UserCreateV1(message.Message):
    """The message sent when a user is created"""

    topic = "fas.user.create"
    body_schema = {
        "id": "http://fedoraproject.org/message-schema/noggin",
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "The message sent when a user is created",
        "type": "object",
        "required": ["msg"],
        "properties": {
            "msg": {
                "required": ["agent", "user"],
                "description": "the contents of the event",
                "type": "object",
                "properties": {
                    "agent": {"type": "string"},
                    "user": {"type": "string"},
                },
            }
        },
    }


class UserUpdateV1(message.Message):
    """The message sent when a user is updated"""

    topic = "fas.user.update"
    body_schema = {
        "id": "http://fedoraproject.org/message-schema/noggin",
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "The message sent when a user is updated",
        "type": "object",
        "required": ["msg"],
        "properties": {
            "msg": {
                "required": ["agent", "user", "fields"],
                "description": "the contents of the event",
                "type": "object",
                "properties": {
                    "agent": {"type": "string"},
                    "user": {"type": "string"},
                    "fields": {"type": "array", "contains": {"type": "string",},},
                },
            },
        },
    }



from noggin_messages import MemberSponsorV1, UserCreateV1, UserUpdateV1
from fedora_messaging import message
import json


class TestSchema:
    """Unit tests for testing the noggin message schemas."""

    def test_member_sponsor_v1(self):
        """
        Test MemberSponsorV1

        The expected message here is:

        {
        "body": {
            "msg": {
            "agent": "dudemcpants",
            "group": "developers",
            "user": "testuser"
            }
        },
        "headers": {
            "fedora_messaging_schema": "noggin.group.member.sponsor.v1",
            "fedora_messaging_severity": 20,
            "sent-at": "2020-03-02T08:53:38+00:00"
        },
        "id": "c795df0d-3a95-47a9-85c4-7fabf3129ddf",
        "queue": null,
        "topic": "fas.group.member.sponsor"
        }

        """

        msg = MemberSponsorV1(
            {"msg": {"agent": "dudemcpants", "user": "testuser", "group": "developers"}}
        )
        msg.validate()

        msg_dump = json.loads(message.dumps(msg))
        assert msg_dump["body"]["msg"]["agent"] == "dudemcpants"
        assert msg_dump["body"]["msg"]["user"] == "testuser"
        assert msg_dump["body"]["msg"]["group"] == "developers"
        assert (
            msg_dump["headers"]["fedora_messaging_schema"]
            == "noggin.group.member.sponsor.v1"
        )
        assert msg_dump["topic"] == "fas.group.member.sponsor"

    def test_user_create_v1(self):
        """
        Test UserCreateV1

        The expected message here is:

        {
        "body": {
            "msg": {
            "agent": "dudemcpants",
            "user": "testuser"
            }
        },
        "headers": {
            "fedora_messaging_schema": "noggin.user.create.v1",
            "fedora_messaging_severity": 20,
            "sent-at": "2020-03-02T08:53:38+00:00"
        },
        "id": "c795df0d-3a95-47a9-85c4-7fabf3129ddf",
        "queue": null,
        "topic": "fas.user.create"
        }

        """

        msg = UserCreateV1({"msg": {"agent": "dudemcpants", "user": "testuser"}})
        msg.validate()

        msg_dump = json.loads(message.dumps(msg))
        assert msg_dump["body"]["msg"]["agent"] == "dudemcpants"
        assert msg_dump["body"]["msg"]["user"] == "testuser"
        assert msg_dump["headers"]["fedora_messaging_schema"] == "noggin.user.create.v1"
        assert msg_dump["topic"] == "fas.user.create"

    def test_user_update_v1(self):
        """
        Test UserUpdateV1

        The expected message here is:

        {
        "body": {
            "msg": {
            "agent": "dudemcpants",
            "user": "testuser",
            "fields": ["firstname", "lastname", "gpgkeyid"]
            }
        },
        "headers": {
            "fedora_messaging_schema": "noggin.user.update.v1",
            "fedora_messaging_severity": 20,
            "sent-at": "2020-03-02T08:53:38+00:00"
        },
        "id": "c795df0d-3a95-47a9-85c4-7fabf3129ddf",
        "queue": null,
        "topic": "fas.user.update"
        }

        """

        msg = UserUpdateV1(
            {
                "msg": {
                    "agent": "dudemcpants",
                    "user": "testuser",
                    "fields": ["firstname", "lastname", "gpgkeyid"],
                }
            }
        )
        msg.validate()

        msg_dump = json.loads(message.dumps(msg))
        assert msg_dump["body"]["msg"]["agent"] == "dudemcpants"
        assert msg_dump["body"]["msg"]["user"] == "testuser"
        assert msg_dump["body"]["msg"]["fields"] == [
            "firstname",
            "lastname",
            "gpgkeyid",
        ]
        assert msg_dump["headers"]["fedora_messaging_schema"] == "noggin.user.update.v1"
        assert msg_dump["topic"] == "fas.user.update"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['noggin_messages', 'noggin_messages.tests']

package_data = \
{'': ['*']}

install_requires = \
['fedora-messaging>=2.0.1,<3.0.0']

entry_points = \
{'fedora.messages': ['noggin.group.member.sponsor.v1 = '
                     'noggin_messages:MemberSponsorV1',
                     'noggin.user.create.v1 = noggin_messages:UserCreateV1',
                     'noggin.user.update.v1 = noggin_messages:UserUpdateV1']}

setup_kwargs = {
    'name': 'noggin-messages',
    'version': '0.0.1',
    'description': 'Fedora Messaging message schemas for Noggin.',
    'long_description': None,
    'author': 'Fedora Infrastructure',
    'author_email': 'admin@fedoraproject.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/fedora-infra/noggin-messages',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
